;***************************************************************************
;* project         : MAHPONG 	                                           *
;*-------------------------------------------------------------------------*
;* program         : SOCCER game submodule				   *
;***************************************************************************
;* file            : SOCCER.inc                              		   *
;* version         : 1.0a                                                  *
;*-------------------------------------------------------------------------*
;* original Date   : 03.05.17    Author: M. Hasenstab                      *
;*                 :                     marcus.hasenstab@jkdesign.de      *
;*-------------------------------------------------------------------------*
;* last change     : 03.05.17    Author: MAH		                   *
;***************************************************************************
;* destination HW  : 8515 plus 2bitvideo DAC	                           *
;* destination SW  : 						           *
;* assembler       : wavrasm					           *
;* language        : assembler for AVR		                           *
;***************************************************************************
;*                      COPYRIGHT (c)  				           *
;***************************************************************************

;***************************************************************************
;*                      SOCCER VARIABLES                                   *
;***************************************************************************


;***************************************************************************
;*                      SOCCER CONSTANTS                                   *
;***************************************************************************
.equ	SOCCER_SPEED	= 1
.equ	SOCCER_GOAL1X	= 0
.equ	SOCCER_GOAL2X	= 39
.equ	SOCCER_SCOREWIN = 21

.equ	SOCCER_BATX1	= 3;
.equ	SOCCER_BATX2	= 40-4;
.equ	SOCCER_BATYMIN	= 0+3;
.equ	SOCCER_BATYMAX	= 29-4;
.equ	SOCCER_BATXMIN	= 0+3;
.equ	SOCCER_BATXMAX	= 40-4;


;***************************************************************************
;*                      SOCCER MENU                                        *
;***************************************************************************
SOCCER_MENU:

SOCCER_MENU_PLOTSCREEN:
	sbrs	FLAG_I, 6			; redraw on new menu only
	rjmp	SOCCER_MENU_PLOTSCREEN_END
	ldi	ZL, low( SOCCER_SCREEN_MENU*2 )	; plot menu screen
	ldi	ZH, high( SOCCER_SCREEN_MENU*2 )
	rcall	PLOTSCREEN
	cbr	FLAG_I, (1<<6)
SOCCER_MENU_PLOTSCREEN_END:
	
SOCCER_MENU_CHECKSTART:
	sbic	PADDLEPIN, PADDLE1UP
	ret
	sbic	PADDLEPIN, PADDLE2UP
	ret

SOCCER_MENU_STARTGAME:
	rjmp	SOCCER_GAME_SETUP
	ret


;***************************************************************************
;*                      SOCCER GAME CODE                                   *
;***************************************************************************
SOCCER_GAME_SETUP:
	ldi	ZL, low( SOCCER_COURT*2 )	; plot court screen
	ldi	ZH, high( SOCCER_COURT*2 )
	rcall	PLOTSCREEN
	
	ldi	VREG1, 45			; draw winning score marker
	ldi	VREG2, 29-SOCCER_SCOREWIN
	ldi	TMP, 0x03
	rcall	SETPIXEL
	ldi	VREG1, 46
	ldi	VREG2, 29-SOCCER_SCOREWIN
	ldi	TMP, 0x03
	rcall	SETPIXEL

	ldi	TMPL, SOCCER_SPEED		; start speed x
	sbrc	TICKL, 1
	ldi	TMPL, -SOCCER_SPEED
	ldi	TMPH, SOCCER_SPEED		; start speed y
	sbrc	TICKL, 2
	ldi	TMPH, -SOCCER_SPEED

	ldi	TMP, 80				; start coordinates
	ldi	YL, low(BALL_X)			; set ball coordinates in SRAM
	ldi	YH, high(BALL_X)
	st	Y, TMP
	ldi	TMP, 10
	ldi	YL, low(BALL_Y)
	ldi	YH, high(BALL_Y)
	st	Y, TMP
	
	ldi	YL, low(BALLSPEED_X)		; set ball speed in SRAM
	ldi	YH, high(BALLSPEED_X)
	st	Y, TMPL
	ldi	YL, low(BALLSPEED_Y)
	ldi	YH, high(BALLSPEED_Y)
	st	Y, TMPH
	
	ldi	YL, low(BAT1_Y)	; get paddle coordinates from SRAM
	ldi	YH, high(BAT1_Y)
	ldi	TMP, SOCCER_BATYMAX/2
	st	Y, TMP
	ldi	YL, low(BAT2_Y)	; get paddle coordinates from SRAM
	ldi	YH, high(BAT2_Y)
	ldi	TMP, SOCCER_BATYMAX/2
	st	Y, TMP
	ldi	YL, low(BAT1_X)	; get paddle coordinates from SRAM
	ldi	YH, high(BAT1_X)
	ldi	TMP, SOCCER_BATXMIN
	st	Y, TMP
	ldi	YL, low(BAT2_X)	; get paddle coordinates from SRAM
	ldi	YH, high(BAT2_X)
	ldi	TMP, SOCCER_BATXMAX
	st	Y, TMP

	ldi	YL, low(SCORE1)	; get paddle coordinates from SRAM
	ldi	YH, high(SCORE1)
	ldi	TMP, 0
	st	Y, TMP
	ldi	YL, low(SCORE2)	; get paddle coordinates from SRAM
	ldi	YH, high(SCORE2)
	ldi	TMP, 0
	st	Y, TMP
	

SOCCER_GAME_LOOP:
	rcall	WAIT_VSYNC	; synchronise game with screen
	rcall	WAIT_VSYNC	; synchronise game with screen
	rcall	SOCCER_DOPADDLE	; do paddle
	
	
;PLAYPONG_GETBALL:
	ldi	YL, low(BALL_X)	; get ball coordinates from SRAM
	ldi	YH, high(BALL_X)
	ld	LREG1, Y
	ldi	YL, low(BALL_Y)
	ldi	YH, high(BALL_Y)
	ld	LREG2, Y
	mov	LREG5, LREG1		; old ball position
	mov	LREG6, LREG2
	ldi	YL, low(BALLSPEED_X)	; get ball speed from SRAM
	ldi	YH, high(BALLSPEED_X)
	ld	LREG3, Y
	ldi	YL, low(BALLSPEED_Y)
	ldi	YH, high(BALLSPEED_Y)
	ld	LREG4, Y

;PLAYPONG_CLEARCURRENTBALL_POSITION:
	mov	VREG1, LREG5	; clear ball position on screen
	mov	VREG2, LREG6	
	lsr	VREG1		; the resolution in SRAM is 4times higher than on the screen
	lsr	VREG1		;  thus the shifting
	lsr	VREG2
	lsr	VREG2
	ldi	TMP, 0x00
	rcall	SETPIXEL
	
;PLAYPONG_CALC_NEW_BALL_POSITION:
SOCCER_PLAYPONG_CNBP:
	add	LREG1, LREG3	; add x-speed to x-pos
	mov	VREG1, LREG1	; check for obstacle on new position
	mov	VREG2, LREG2	
	lsr	VREG1		
	lsr	VREG1		
	lsr	VREG2
	lsr	VREG2

	rcall	SOCCER_DOSCORE

	rcall	GETPIXEL	; GETPIXEL returns 1 if pixel set
	andi	TMP, 0x03
	breq	SOCCER_PLAYPONG_CNBP_X1 ; no obstacle, go on!
	neg	LREG3		; -> obstacle in x direction -> neg x-speed
	add	LREG1, LREG3
	
	ldi	TMPH, high(SOCCER_SOUND_BAT1*2)
	ldi	TMPL, low(SOCCER_SOUND_BAT1*2)
	rcall	STARTSOUND

	rjmp	SOCCER_PLAYPONG_CNBP
SOCCER_PLAYPONG_CNBP_X1:

	add	LREG2, LREG4	; add y-speed to y-pos
	mov	VREG1, LREG1	; check for obstacle on new position
	mov	VREG2, LREG2	
	lsr	VREG1		
	lsr	VREG1		
	lsr	VREG2
	lsr	VREG2
	rcall	GETPIXEL	; GETPIXEL returns 1 if pixel set
	andi	TMP, 0x03
	breq	SOCCER_PLAYPONG_CNBP_Y1 ; no obstacle, go on!
	neg	LREG4		; -> obstacle in y direction -> neg y-speed
	add	LREG2, LREG4

	ldi	TMPH, high(SOCCER_SOUND_BAT1*2)
	ldi	TMPL, low(SOCCER_SOUND_BAT1*2)
	rcall	STARTSOUND

	rjmp	SOCCER_PLAYPONG_CNBP
SOCCER_PLAYPONG_CNBP_Y1:

;PLAYPONG_SETCURRENTBALL_POSITION:
	mov	VREG1, LREG1	; set ball position on screen
	mov	VREG2, LREG2	
	lsr	VREG1		; the resolution in SRAM is 4times higher than on the screen
	lsr	VREG1		;  thus the shifting
	lsr	VREG2
	lsr	VREG2
	ldi	TMP, 0x03
	rcall	SETPIXEL

;PLAYPONG_SETBALL:
	ldi	YL, low(BALL_X)		; set ball coordinates in SRAM
	ldi	YH, high(BALL_X)
	st	Y, LREG1
	ldi	YL, low(BALL_Y)
	ldi	YH, high(BALL_Y)
	st	Y, LREG2
	ldi	YL, low(BALLSPEED_X)	; set ball speed in SRAM
	ldi	YH, high(BALLSPEED_X)
	st	Y, LREG3
	ldi	YL, low(BALLSPEED_Y)
	ldi	YH, high(BALLSPEED_Y)
	st	Y, LREG4
	rjmp	SOCCER_GAME_LOOP

	
	
; do paddle calculation and drawing
SOCCER_DOPADDLE:
	ldi	YL, low(BAT1_Y)	; get paddle coordinates from SRAM
	ldi	YH, high(BAT1_Y)
	ld	VREG2, Y
	ldi	YL, low(BAT1_X)	; get paddle coordinates from SRAM
	ldi	YH, high(BAT1_X)
	ld	VREG1, Y

SOCCER_DOPADDLE1_ERASE:
	ldi	TMP, 0x00	; color black
	dec	VREG2
	rcall	SETPIXEL
	inc	VREG2
	rcall	SETPIXEL
	inc	VREG2
	rcall	SETPIXEL
	inc	VREG2
	rcall	SETPIXEL
	dec	VREG2
	dec	VREG2
	sbis	PADDLEPIN, PADDLE1UP	; paddle movement
	dec	VREG2
	sbis	PADDLEPIN, PADDLE1DOWN
	inc	VREG2
	cpi	VREG2, SOCCER_BATYMIN		; check upper limit
	brge	SOCCER_DOPADDLE1_1
	ldi	VREG2, SOCCER_BATYMIN
SOCCER_DOPADDLE1_1:
	cpi	VREG2, SOCCER_BATYMAX		; check upper limit
	brlt	SOCCER_DOPADDLE1_2
	ldi	VREG2, SOCCER_BATYMAX
SOCCER_DOPADDLE1_2:
	ldi	YL, low(BAT1_Y)		; set paddle y coordinate in SRAM
	ldi	YH, high(BAT1_Y)
	st	Y, VREG2

	sbis	PADDLEPIN, PADDLE1LEFT	; paddle movement
	dec	VREG1
	sbis	PADDLEPIN, PADDLE1RIGHT
	inc	VREG1
	cpi	VREG1, SOCCER_BATXMIN		; check left limit
	brge	SOCCER_DOPADDLE1_3
	ldi	VREG1, SOCCER_BATXMIN
SOCCER_DOPADDLE1_3:
	cpi	VREG1, SOCCER_BATXMAX		; check right limit
	brlt	SOCCER_DOPADDLE1_4
	ldi	VREG1, SOCCER_BATXMAX
SOCCER_DOPADDLE1_4:
	ldi	YL, low(BAT1_X)		; set paddle coordinates in SRAM
	ldi	YH, high(BAT1_X)
	st	Y, VREG1


SOCCER_DOPADDLE1_DRAW:
	ldi	TMP, 0x02		; color light grey
	dec	VREG2
	rcall	SETPIXEL
	inc	VREG2
	rcall	SETPIXEL
	inc	VREG2
	rcall	SETPIXEL
	inc	VREG2
	rcall	SETPIXEL

SOCCER_DOPADDLE2:
	ldi	YL, low(BAT2_Y)	; get paddle coordinates from SRAM
	ldi	YH, high(BAT2_Y)
	ld	VREG2, Y
	ldi	YL, low(BAT2_X)	; get paddle coordinates from SRAM
	ldi	YH, high(BAT2_X)
	ld	VREG1, Y
SOCCER_DOPADDLE2_ERASE:
	ldi	TMP, 0x00	; color black
	dec	VREG2
	rcall	SETPIXEL
	inc	VREG2
	rcall	SETPIXEL
	inc	VREG2
	rcall	SETPIXEL
	inc	VREG2
	rcall	SETPIXEL
	dec	VREG2
	dec	VREG2
	sbis	PADDLEPIN, PADDLE2UP	; paddle movement
	dec	VREG2
	sbis	PADDLEPIN, PADDLE2DOWN
	inc	VREG2
	cpi	VREG2, SOCCER_BATYMIN		; check upper limit
	brge	SOCCER_DOPADDLE2_1
	ldi	VREG2, SOCCER_BATYMIN
SOCCER_DOPADDLE2_1:
	cpi	VREG2, SOCCER_BATYMAX		; check upper limit
	brlt	SOCCER_DOPADDLE2_2
	ldi	VREG2, SOCCER_BATYMAX
SOCCER_DOPADDLE2_2:
	ldi	YL, low(BAT2_Y)		; set paddle y coordinate in SRAM
	ldi	YH, high(BAT2_Y)
	st	Y, VREG2

	sbis	PADDLEPIN, PADDLE2LEFT	; paddle movement
	dec	VREG1
	sbis	PADDLEPIN, PADDLE2RIGHT
	inc	VREG1
	cpi	VREG1, SOCCER_BATXMIN		; check left limit
	brge	SOCCER_DOPADDLE2_3
	ldi	VREG1, SOCCER_BATXMIN
SOCCER_DOPADDLE2_3:
	cpi	VREG1, SOCCER_BATXMAX		; check right limit
	brlt	SOCCER_DOPADDLE2_4
	ldi	VREG1, SOCCER_BATXMAX
SOCCER_DOPADDLE2_4:
	ldi	YL, low(BAT2_X)		; set paddle coordinates in SRAM
	ldi	YH, high(BAT2_X)
	st	Y, VREG1

SOCCER_DOPADDLE2_DRAW:
	ldi	TMP, 0x01		; color dark grey
	dec	VREG2
	rcall	SETPIXEL
	inc	VREG2
	rcall	SETPIXEL
	inc	VREG2
	rcall	SETPIXEL
	inc	VREG2
	rcall	SETPIXEL
	ret
	

SOCCER_DOSCORE:
	push	TMP
	push	VREG1
	push	VREG2

	ldi	YL, low(SCORE2)	; get data from SRAM
	ldi	YH, high(SCORE2)
	ld	TMP, Y
	cpi	VREG1, SOCCER_GOAL1X+1	; if goal
	brge	SOCCER_DOSCORE_1
	inc	TMP		; -> inc score
	st	Y, TMP

	cpi	TMP, SOCCER_SCOREWIN
	brge	SOCCER_DOSCORE_MATCH2

	ldi	VREG1, 39+4+3
	ldi	VREG2, 29
	sub	VREG2, TMP
	ldi	TMP, 0x01
	rcall	SETPIXEL
	
	rcall	STOPSOUND
	ldi	TMPH, high(SOCCER_SOUND_GOAL1*2)
	ldi	TMPL, low(SOCCER_SOUND_GOAL1*2)
	rcall	STARTSOUND
	
	rjmp	SOCCER_DOSCORE_2	
SOCCER_DOSCORE_1:

SOCCER_DOSCORE21:
	ldi	YL, low(SCORE1)	; get data from SRAM
	ldi	YH, high(SCORE1)
	ld	TMP, Y
	cpi	VREG1, SOCCER_GOAL2X	; if goal
	brlt	SOCCER_DOSCORE_2
	inc	TMP		; -> inc score
	st	Y, TMP

	cpi	TMP, SOCCER_SCOREWIN
	brge	SOCCER_DOSCORE_MATCH1

	ldi	VREG1, 39+4+2
	ldi	VREG2, 29
	sub	VREG2, TMP
	ldi	TMP, 0x02
	rcall	SETPIXEL
	
	rcall	STOPSOUND
	ldi	TMPH, high(SOCCER_SOUND_GOAL2*2)
	ldi	TMPL, low(SOCCER_SOUND_GOAL2*2)
	rcall	STARTSOUND
SOCCER_DOSCORE_2:	
	
	pop	VREG2
	pop	VREG1
	pop	TMP
	ret	

SOCCER_DOSCORE_MATCH1:
	push	ZL
	push	ZH
	ldi	ZL, low( SCREEN_PLAYER1WINS*2 )
	ldi	ZH, high( SCREEN_PLAYER1WINS*2 )
	rcall	PLOTSCREEN

	rcall	STOPSOUND
	ldi	TMPH, high(SOUND_PLAYER1WINS*2)
	ldi	TMPL, low(SOUND_PLAYER1WINS*2)
	rcall	STARTSOUND

	pop	ZL
	pop	ZH
SOCCER_DOSCORE_MATCH_FE:
	rcall	WAIT_VSYNC		
	sbic	PADDLEPIN, PADDLE1DOWN
	rjmp	SOCCER_DOSCORE_MATCH_FE
	sbic	PADDLEPIN, PADDLE2DOWN
	rjmp	SOCCER_DOSCORE_MATCH_FE
	rjmp	RESET
;	rcall	STOPSOUND
;	sbr	FLAG_I, (1<<6)			; redraw menu
;	rjmp	SOCCER_MENU

SOCCER_DOSCORE_MATCH2:
	push	ZL
	push	ZH
	ldi	ZL, low( SCREEN_PLAYER2WINS*2 )
	ldi	ZH, high( SCREEN_PLAYER2WINS*2 )
	rcall	PLOTSCREEN

	rcall	STOPSOUND
	ldi	TMPH, high(SOUND_PLAYER2WINS*2)
	ldi	TMPL, low(SOUND_PLAYER2WINS*2)
	rcall	STARTSOUND

	pop	ZL
	pop	ZH
	rjmp	SOCCER_DOSCORE_MATCH_FE



;***************************************************************************
;*                      SOCCER VIDEO PATTERNS                              *
;***************************************************************************

SOCCER_SCREEN_MENU:
	.db 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F
	.db 0x78, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xE1
	.db 0x7B, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xED
	.db 0x7A, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xE5
	.db 0x7A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE5
	.db 0x7A, 0x30, 0xC0, 0xE0, 0x30, 0x90, 0xD0, 0xE0, 0xF0, 0x00, 0x00, 0xE5
	.db 0x7A, 0x40, 0x10, 0x10, 0x40, 0x20, 0x10, 0x00, 0x80, 0x80, 0x00, 0xE5
	.db 0x7A, 0x40, 0x10, 0x10, 0x40, 0x20, 0x10, 0x00, 0x80, 0x80, 0x00, 0xE5
	.db 0x7A, 0x30, 0x90, 0x10, 0x40, 0x20, 0x10, 0xC0, 0xF0, 0x01, 0x08, 0xE5
	.db 0x7A, 0x00, 0x50, 0x10, 0x40, 0x20, 0x10, 0x00, 0xA0, 0x01, 0x08, 0xE5
	.db 0x7A, 0x00, 0x50, 0x10, 0x40, 0x20, 0x10, 0x00, 0x90, 0x01, 0x08, 0xE5
	.db 0x7A, 0x70, 0x80, 0xE0, 0x30, 0x90, 0xD0, 0xE0, 0x80, 0x90, 0x80, 0xE5
	.db 0x7A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x12, 0x84, 0xE5
	.db 0x7A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x16, 0x86, 0xE5
	.db 0x7A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x16, 0x86, 0xE5
	.db 0x7A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x12, 0x84, 0xE5
	.db 0x7A, 0x03, 0x09, 0x0C, 0x00, 0x0E, 0x03, 0x0B, 0x0E, 0x10, 0x80, 0xE5
	.db 0x7A, 0x04, 0x05, 0x02, 0x00, 0x09, 0x04, 0x04, 0x08, 0x01, 0x08, 0xE5
	.db 0x7A, 0x00, 0x05, 0x01, 0x00, 0x09, 0x04, 0x04, 0x0B, 0x09, 0x08, 0xE5
	.db 0x7A, 0x00, 0x09, 0x01, 0x06, 0x0E, 0x07, 0x0C, 0x0A, 0x01, 0x08, 0xE5
	.db 0x7A, 0x01, 0x01, 0x01, 0x00, 0x09, 0x04, 0x04, 0x09, 0x00, 0x00, 0xE5
	.db 0x7A, 0x02, 0x01, 0x02, 0x00, 0x09, 0x04, 0x04, 0x08, 0x08, 0x00, 0xE5
	.db 0x7A, 0x07, 0x0D, 0x0C, 0x00, 0x0E, 0x04, 0x04, 0x0B, 0x08, 0x00, 0xE5
	.db 0x7A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE5
	.db 0x7A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE5
	.db 0x7A, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xE5
	.db 0x7A, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xE5
	.db 0x7B, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xED
	.db 0x78, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xE1
	.db 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F


;***************************************************************************
;*                      SOCCER SOUND SCORES                                *
;***************************************************************************
SOCCER_SOUND_BAT1:
	.db tC1,4,0,0
SOCCER_SOUND_BAT2:
	.db tG1,4,0,0	

SOCCER_SOUND_GOAL1:
	.db tC1,4
	.db tD1,4
	.db tE1,4
	.db 0,0	
SOCCER_SOUND_GOAL2:
	.db tG1,4
	.db tA1,4
	.db tH1,4
	.db 0,0	
